require('dotenv').config()

const config = require('../config')

const hre = require('hardhat')

const { ethers } = hre

const { createInterface } = require('readline')

const prompter = createInterface({ input: process.stdin, output: process.stdout })

function _prompt(prompt, resolve) {
  prompter.question(prompt, (answer) => {
    if (answer == 'y') {
      resolve(true)
    } else if (answer == 'n') {
      resolve(false)
    } else _prompt('', resolve)
  })
}

function prompt(prompt) {
  return new Promise((resolve) => _prompt(prompt, resolve))
}

function deployedMessage(name, chainId, address) {
  return `\n${name} deployed on ${idToNetwork(chainId)} @ ${address}\n`
}

function verifiedMessage(name, address) {
  return `\n${name} @ ${address} verified on Etherscan!\n`
}

function timeout(seconds) {
  return new Promise((resolve) => setTimeout(resolve, seconds * 1000))
}

const promptMessageBase = (middle) => `\n${middle}\n\nAre you sure you would like to continue? (y/n): `

async function deploy() {
  const signer = await ethers.getSigner()

  const stakingDeployer = (await ethers.getContractFactory('TornadoStakingRewards')).connect(signer)

  let staking

  if (await prompt(promptMessageBase('Continuing to TornadoStakingRewards deployment.'))) {
    staking = await stakingDeployer.deploy(config.governance, config.TORN, config.registry)
    console.log(deployedMessage('TornadoStakingRewards', 1, staking.address))
  } else {
    return '\nDecided to stop at TornadoStakingRewards deployment.\n'
  }

  if (await prompt(promptMessageBase('Continuing to contract verification.'))) {
    await hre.run('verify:verify', {
      address: staking.address,
      constructorArguments: [config.governance, config.TORN, config.registry],
    })
    console.log(verifiedMessage('TornadoStakingRewards'))
  } else {
    return '\nDecided to stop at contract verification.\n'
  }
}

deploy()
